<?PHP
/*
This package can be used to implement Aspect Oriented Programming (AOP, 
http://en.wikipedia.org/wiki/Aspect-oriented_programming) by executing 
the code of classes that enable orthogonal aspects at run-time.

The intention is to provide a means implement orthogonal aspects in separate 
classes that may be interesting add to the application, like logging, caching, 
transaction control, etc., without affecting the main business logic.

The package provides base classes for implementing defining point cuts where 
the code of advice class is called to implement actions of the orthogonal aspects 
that an application may need to enable.

Copyright (c) Dmitry Sheiko http://www.cmsdevelopment.com
*/
include("aop.lib.php");
$aspect1 = new Aspect();
$pc1 = $aspect1->pointcut("call Sample::Sample  or call Sample::Sample2");
$pc1->_before("print 'Aspect1 preprocessor<br />';");
$pc1->_after("print 'Aspect1 postprocessor<br />';");
$pc1->destroy();
$aspect2 = new Aspect();
$pc2 = $aspect2->pointcut("call *::Sample2");
$pc2->_before("print 'Aspect2 preprocessor<br />';");
$pc2->_after("print 'Aspect2 postprocessor<br />';");
$pc2->destroy();
$aspect3 = new Aspect();
$pc3 = $aspect3->pointcut("call *::*");
$pc3->_event("SomeEvent", "print 'Aspect3 specified event<br />';");
$pc3->destroy();

Aspect::apply($aspect1);
Aspect::apply($aspect2);
Aspect::apply($aspect3);

Class Sample {
	function Sample() {
		Advice::_before();
		Advice::_event("SomeEvent");
		print 'Some business logic of Sample<br />';
		Advice::_after();
	}
	function Sample2() {
		Advice::_before();
		print 'Some business logic of Sample2<br />';
		Advice::_after();
	}
	
}

$Sample = new Sample();
$Sample->Sample2();
?>